from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.Archetypes.atapi import ComputedField, MultiSelectionWidget
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import Schema, registerType
from Products.Archetypes.atapi import StringField, StringWidget
from Products.CMFCore.utils import getToolByName
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from ftw.tabbedview.interfaces import ITabbedView
from izug.ticketbox import ticketboxMessageFactory as _
from izug.ticketbox.config import PROJECTNAME
from izug.ticketbox.interfaces import ITicketBox
from zope.component import queryUtility, getUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory


TicketBoxSchema = folder.ATBTreeFolderSchema.copy() + Schema((

        #Individual Identifier
        StringField(
            name='individualIdentifier',
            searchable=True,

            widget=StringWidget(
                maxlength=7,
                label=_(u"label_individualIdentifier",
                        default=u"Individual identifier"),
                description=_(
                    u"help_individualIdentifier",
                    default=u"Enter a individual identifier " +
                    u"(max 7 positions)"))),

        #Available States
        DataGridField(
            name='availableStates',
            searchable=True,
            allow_empty_rows=False,

            default=(
                {'id': '',
                 'title': "offen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "in Bearbeitung",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "zur\xc3\xbcckgewiesen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "zum Testen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "erledigt",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '0',
                 },

                {'id': '',
                 'title': "verschoben",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },
                ),

            columns=("id",
                     "title",
                     "show_in_all_tickets",
                     "show_in_my_tickets"),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u"Define states"),
                description=_(u"add or delete possible state-information"),

                columns={
                    'id': Column(_(u"id")),
                    'title': Column(_(u"title")),
                    'show_in_all_tickets': SelectColumn(
                        _(u"show in 'all tickets'"),
                        # DatagridFields SelectColumn doesn not behave like an
                        # archetype field. It always does
                        # a getattr to get the vocab.
                        vocabulary="yes_no"),
                    'show_in_my_tickets': SelectColumn(
                        _(u"show in 'my tickets'"),
                        vocabulary="yes_no")})),

        #Available Releases
        DataGridField(
            name='availableReleases',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_availableReleases',
                        default=u"Available Releases"),
                description=_(u'help_avialableReleases',
                              default=u"Enter the Available Releases " +
                              u"for this tracker."),
                column_names=(_(u'Releases_id'), _(u'Releases_title')))),

        #Available Priorities
        DataGridField(
            name='availablePriorities',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_availablePriorities',
                        u"Available priorities"),
                description=_(u'help_availablePriorities',
                              default=u"Enter the different type of " +
                              u"issue severities that should be " +
                              u"available, one per line."),
                column_names=(
                    _(u'Priorities_id'),
                    _(u'Priorities_title')))),

        #Available Areas
        DataGridField(
            name='availableAreas',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_Areas', default=u"Areas"),
                description=_(u'help_areas',
                              default=u"Enter the issue topics/areas" +
                              u" for this tracker."),
                column_names=(
                    _(u'Areas_id'),
                    _(u'Areas_title')))),

        #Available Varieties
        DataGridField(
            name='availableVarieties',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                label=_(u'label_Varieties', default=u"Varieties"),
                description=_(u'help_varieties',
                              default=u"Enter the issue varieties for " +
                                  u"this tracker."),
                column_names=(
                    _(u'Varieties_id'),
                    _(u'Varieties_title')))),

        ComputedField(
            name='assignable_user_ids',
            mode='wr',
            accessor='getAssignableUserIds',
            edit_accessor='getAssignableUserIds',
            mutator='setAssignableUserIds',
            vocabulary='unfilteredAssignableUsersVocabulary',
            expression='context.getAssignableUserIds()',

            widget=MultiSelectionWidget(
                format='checkbox',
                label=_(u'label_assignable_users',
                        default=u'Assignable users'),
                description=_(
                    u'help_assignable_users',
                    default=u'Select the users which should be assignable. '
                    'New users are by default searchable.'))),

        LinesField(
            name='not_assignable_user_ids',
            widget=MultiSelectionWidget(
                modes=())),

        ))


schemata.finalizeATCTSchema(
    TicketBoxSchema,
    folderish=True,
    moveDiscussion=False,
    )


class TicketBox(folder.ATBTreeFolder):
    """A tracker-like task management system"""

    implements(ITicketBox, ITabbedView)

    meta_type = "TicketBox"
    schema = TicketBoxSchema
    security = ClassSecurityInfo()

    def assignable_users(self):
        """
        Get the managers available as a DisplayList. The first item is 'None',
        with a key of '(UNASSIGNED)'.
        """

        factory = queryUtility(IVocabularyFactory, name='assignable_users')
        if factory is None:
            factory = getUtility(IVocabularyFactory,
                                 name='plone.principalsource.Users',
                                 context=self)

        vocabulary = factory(self)

        # TODO: Use vocabulary instead of list of tuples.
        users = [('(UNASSIGNED)', _(u'None'))]

        mtool = getToolByName(self, 'portal_membership')
        assignable_userids = self.getAssignableUserIds()

        for term in vocabulary:
            member = mtool.getMemberById(term.token)
            if member is None:
                continue
            if member.getId() not in assignable_userids:
                continue
            user = member.getUser()
            #We do this the same way plone.api does.
            #  But can't use plone.api due to a bug.
            old_security_manager = getSecurityManager()
            newSecurityManager(self.REQUEST, user)
            has_permission = user.checkPermission(
                'izug.ticketbox: Add Ticket',
                self
                )
            setSecurityManager(old_security_manager)
            if member and has_permission:
                title = term.title
                if not title:
                    title = term.value
                users.append((term.token, title))

        return users

    def getAssignableUserIds(self):
        """Returns a list of userids which are currently assignable.
        This respects the configuration whether the assigning of the user
        is enabled.
        """

        not_assignable = self.Schema().getField(
            'not_assignable_user_ids').get(self)

        userids = []
        for userid, title in self.unfilteredAssignableUsersVocabulary():
            if userid not in not_assignable:
                userids.append(userid)
        return userids

    def setAssignableUserIds(self, userids):
        """Select users from the assignable_users vocabulary which should by
        selectable.
        This works by storing the ones which are not selectable for ensuring
        that new ones automatically are selectable.
        """

        assignable_userids = userids or []

        not_assignable_userids = []
        for userid, title in self.unfilteredAssignableUsersVocabulary():
            if userid not in assignable_userids:
                not_assignable_userids.append(userid)

        self.Schema().getField('not_assignable_user_ids').set(
            self, not_assignable_userids)

    def unfilteredAssignableUsersVocabulary(self):
        """Returns a list of userid / title tuples with all users which
        are possible assignable users. It is not filtered by the
        configuration whether the users are currently enabled assignable
        users.
        """
        factory = queryUtility(IVocabularyFactory, name='assignable_users')

        if factory is None:
            factory = getUtility(IVocabularyFactory,
                                 name='plone.principalsource.Users',
                                 context=self)
            terms = factory(self)
        else:
            terms = factory(self, membersonly=True)
        values = []
        for term in terms:
            values.append((term.token, term.title))
        return values

    def yes_no(self):
        """return displaylist with two static rows
        contents: yes and no
        """
        return DisplayList((
                ("1", _(u"yes")),
                ("0", _(u"no")),
                ))

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False


registerType(TicketBox, PROJECTNAME)



from ftw.zipexport.browser.available import ZipExportEnabled


class ZipExportDisabled(ZipExportEnabled):
    """
    This view can be used to hide the zip export action menu by overriding
    the view in the zcml for selected interfaces.
    """
    def zipexport_enabled(self):
        return False



from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.interfaces import IWorkspaceLayer
from izug.ticketbox.interfaces import ITicket
from izug.ticketbox.interfaces import ITicketReferenceStartupDirectory
from zope.component import adapts
from zope.interface import implements


class WorkspaceStartupDirectory(object):
    implements(ITicketReferenceStartupDirectory)
    adapts(ITicket, IWorkspaceLayer)

    def __init__(self, ticket, request):
        self.ticket = ticket
        self.request = request

    def get(self):
        url_tool = self.ticket.restrictedTraverse('@@plone_tools').url()
        startup_obj = self.get_workspace() or self.ticket
        return '/'.join(url_tool.getRelativeContentPath(startup_obj))

    def get_workspace(self):
        context = self.ticket
        while context and not IPloneSiteRoot.providedBy(context):
            if IWorkspace.providedBy(context):
                return context
            context = aq_parent(aq_inner(context))



from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from copy import deepcopy
from izug.ticketbox import ticketboxMessageFactory as _
from izug.ticketbox.browser.tabbed.base import BaseTicketListingTab


def collect_filter_information(items, filters):
    """Creates a dict of filter information from items (brains) and a filter
    configuration.

    filters example:

    >>> filters = {
    ...     'getResponsibleManager': {
    ...         'options': [{'id': 'john.doe', 'title': 'John Doe'}],
    ...         'label': _(u'getResponsibleManager')},
    ... }

    """

    filters = deepcopy(filters)

    # cleanup options to general format
    for filterid, filter in filters.items():
        options = []
        for option in filters[filterid]['options']:

            if isinstance(option, dict):
                options.append(deepcopy(option))

            else:
                options.append({'id': option[0],
                                'title': option[1],
                                'original': option})

        filters[filterid]['options'] = options

    # count matches of every option
    filter_options = {}
    for filterid, filter in filters.items():
        filter_options[filterid] = {}
        for option in filter.get('options', []):
            filter_options[filterid][option['id']] = 0

    for item in items:
        for key in filter_options.keys():
            value = getattr(item, key, None)
            if value and value in filter_options[key]:
                filter_options[key][value] += 1

    # update info with matches
    for filterid, options in filter_options.items():
        filters[filterid]['filterid'] = filterid
        for option in filters[filterid]['options']:
            option['matches'] = options[option['id']]

    return filters


class OverviewTab(BaseTicketListingTab):
    """Ticket overview
    """

    template = ViewPageTemplateFile('overview.pt')

    def get_base_query(self):
        filterid = self.request.get('filterid', None)

        if not filterid:
            # Do not display any results when no filter is selected.
            self.depth = 0

        query = super(OverviewTab, self).get_base_query()

        if filterid:
            filtervalue = self.request.get('filtervalue', None)
            query[filterid] = filtervalue

        return query

    def get_filter_info(self):
        """Returns a list of columns containing filters with all necessary data
        for rendering in the template.
        """

        catalog = getToolByName(self.context, 'portal_catalog')
        brains = catalog({'portal_type': ['Ticket'],
                          'path': '/'.join(self.context.getPhysicalPath())})

        filter_data = collect_filter_information(
            brains, self._get_filter_configuration())

        return (
            # column 1
            [filter_data['getResponsibleManager']],

            # column 2
            [filter_data['getState'],
             filter_data['getVariety']],

            # column 3
            [filter_data['getReleases'],
             filter_data['getWatchedRelease']],

            # column 4
            [filter_data['getPriority'],
             filter_data['getArea']])

    def _get_filter_configuration(self):
        filters = {}
        filters['getResponsibleManager'] = {
            'label': _(u'label_heading_by_responsible',
                       default=u'By responsible'),
            'options': sorted(
                self.context.assignable_users(),
                key=lambda item: item[1].lower().decode('utf-8'))}

        filters['getState'] = {
            'label': _(u'label_heading_by_states'),
            'options': self.context.getAvailableStates()}

        available_releases = self.context.getAvailableReleases()
        filters['getReleases'] = {
            'label': _(u'label_heading_by_release'),
            'options': available_releases}

        filters['getWatchedRelease'] = {
            'label': _(u'label_heading_by_watched_release'),
            'options': available_releases}

        filters['getPriority'] = {
            'label': _(u'label_heading_by_severities'),
            'options': self.context.getAvailablePriorities()}

        filters['getArea'] = {
            'label': _(u'label_heading_by_area'),
            'options': self.context.getAvailableAreas()}

        filters['getVariety'] = {
            'label': _(u'label_heading_by_varieties'),
            'options': self.context.getAvailableVarieties()}

        return filters



from ftw.notification.email.interfaces import IEMailRepresentation
from ftw.notification.email.interfaces import ISubjectCreator
from izug.ticketbox.testing import TICKETBOX_FUNCTIONAL_TESTING
from izug.ticketbox.tests import helpers
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from zope.component import getAdapter
import transaction
import unittest2 as unittest


class TestResponseNotification(unittest.TestCase):

    layer = TICKETBOX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        helpers.login_as_manager(self.portal)
        regtool = getToolByName(self.portal, 'portal_registration')
        regtool.addMember('usera', 'usera',
                          properties={'username': 'usera',
                                      'fullname': 'fullnamea',
                                      'email': 'usera@email.com'})
        self.ticketbox = helpers.create_ticketbox(self.portal)
        self.ticketbox.manage_addLocalRoles('usera',
                                           ['Contributor', 'Manager'])
        self.ticket1 = helpers.create_ticket(self.ticketbox)
        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def get_email_subject_prefix_from_propertysheet(self):
        portal_properties = getToolByName(self.portal, 'portal_properties')
        return portal_properties['ftw.notification-properties'].getProperty(
            'notification_email_subject')

    def test_notification_response_author(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                "usera", "usera",))
        self.browser.open(self.ticket1.absolute_url())
        self.browser.getControl("High").selected = True
        self.browser.getControl("Send notification").selected = True
        self.browser.getControl(name="submit").click()
        list_to = self.browser.getControl(name="to_list:list")
        list_to.controls[0].selected = True
        self.browser.getControl(name='form.button.Send').click()
        self.assertIn(
            'A new Answer has been Added by <span>fullnamea</span> in the '
            'Ticketbox',
            self.portal.MailHost.messages[0])

    def test_fullname_on_ticket_notification(self):
        self.ticket1.setResponsibleManager('usera')

        email_representation = getAdapter(self.ticket1, IEMailRepresentation)
        self.assertIn('fullname', email_representation.template(),
            'The fullname of usea should appear in the mail')

    def test_subject_on_ticketbox_without_individual_identifier(self):
        self.ticketbox.setIndividualIdentifier('')
        subject = getAdapter(self.ticketbox, ISubjectCreator)

        expect = '%s %s' % (self.get_email_subject_prefix_from_propertysheet(),
                            self.ticketbox.Title())

        self.assertEquals(expect,
                          subject(self.ticketbox))

    def test_subject_on_ticketbox_with_individual_identifier(self):
        subject = getAdapter(self.ticketbox, ISubjectCreator)
        expect = '%s [%s] %s' % (
            self.get_email_subject_prefix_from_propertysheet(),
            self.ticketbox.getIndividualIdentifier(),
            self.ticketbox.Title())

        self.assertEquals(expect,
                          subject(self.ticketbox))

    def test_subject_on_ticket_without_individual_indentifier(self):
        self.ticketbox.setIndividualIdentifier('')
        subject = getAdapter(self.ticket1, ISubjectCreator)

        expect = '%s #%s - %s' % (
            self.get_email_subject_prefix_from_propertysheet(),
            self.ticket1.getId(),
            self.ticket1.Title())

        self.assertEquals(expect,
                          subject(self.ticket1))

    def test_subject_on_ticket_with_individual_indentifier(self):
        subject = getAdapter(self.ticket1, ISubjectCreator)

        expect = '%s [%s] #%s - %s' % (
            self.get_email_subject_prefix_from_propertysheet(),
            self.ticketbox.getIndividualIdentifier(),
            self.ticket1.getId(),
            self.ticket1.Title())

        self.assertEquals(expect,
                          subject(self.ticket1))



from izug.ticketbox.browser.helper import map_attribute, readable_author
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from DateTime import DateTime

class TicketView(BrowserView):

    template = ViewPageTemplateFile('ticket_view.pt')

    # HELPER Methods for ftw.table generator
    def map_state(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "state")

    def map_priority(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "priority")

    def map_area(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "area")

    def map_variety(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "variety")

    def map_release(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "releases")

    def map_watched_release(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "watchedRelease")

    def map_author(self):
        """
        get the full name of a user-id
        """
        return readable_author(self.context)

    def get_crator(self):
        creator = self.context.Creator()
        user = self.context.acl_users.getUserById(creator)
        if user is None:
            return creator
        else:
            return user.getProperty('fullname', creator)

    def getCreationDate(self):
        context = self.context
        creation_date = context.CreationDate()
        date = DateTime(creation_date)
        return date.strftime('%d.%m.%Y %H:%M')



from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import queryUtility


def uniquify_ids(data):
    """Creates unique ids within a list of dicts.
    Sets the "id" key of each dicts, creates the id with the value of
    "title" within the dict.

    ``data`` example:

    >>> [{'id': '', 'title': 'Foo'},
    ...  {'id': 'bar', 'title': 'Bar'}]
    """

    existing_ids = set([item.get('id') for item in data
                        if item.get('id')])

    for item in data:
        if item.get('id'):
            continue

        item['id'] = create_uniqe_id(item.get('title'), existing_ids)
        existing_ids.add(item['id'])

    return data


def create_uniqe_id(title, existing_ids):
    """Creates a uniqe id by using the IIDNormalizer utility.
    """

    id_ = queryUtility(IIDNormalizer).normalize(title)

    if id_ not in existing_ids:
        return id_

    base_id = id_
    index = 0

    while id_ in existing_ids:
        index += 1
        id_ = '%s-%i' % (base_id, index)

    return id_



# -*- coding: utf-8 -*-
from izug.ticketbox import export
from Products.Five.browser import BrowserView


class TicketsExport(BrowserView):
    """Returns a csv wincp52 ecoded file"""
    def __call__(self):
        response = self.request.RESPONSE
        filename = 'export_%s.csv' % self.context.getId()
        encoding = 'Windows-1252'
        csv = []

        # Header
        header = [field.encode('utf-8') for field
                  in export.get_header(self.context)]
        csv.append(','.join(header))

        # Data
        data = export.get_data(self.context)
        for row in data:
            # Change double quote to single
            row = [val.replace('"', "'") for val in row]

            # Put every value in double quote
            row = ['"%s"' % val for val in row]
            csv.append(','.join(row))

        result = '\n'.join(csv).decode('utf-8').encode(encoding, 'replace')

        response.setHeader('Content-Type',
                           'text/csv; charset=%s' % encoding)
        response.setHeader(
            'Content-disposition',
            'attachment; filename={0}'.format(filename))

        return result


class TicketsExportXlsx(TicketsExport):
    """Returns a XLSX file containing tickets."""

    def __call__(self, *args, **kwargs):
        response = self.request.RESPONSE

        header = export.get_header(self.context)
        data = export.get_data(self.context)
        excel_response = export.create_xlsx(header, data)

        filename = 'export_%s.xlsx' % self.context.getId()

        response.setHeader(
            'Content-Type',
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        )
        response.setHeader(
            'Content-disposition',
            'attachment; filename={0}'.format(filename)
        )

        return excel_response.read()



# pylint: disable=E0211, E0213
# E0211: Method has no argument
# E0213: Method should have "self" as first argument


from zope.contentprovider.interfaces import ITALNamespaceData
from zope.interface import Attribute
from zope.interface import directlyProvides
from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager


class ITicketBox(Interface):
    """A tracker-like task management system"""


class ITicket(Interface):
    """A ticket for the Ticketbox"""


class IResponseContainer(Interface):
    """holds the responses for a ticket"""
    pass


class IResponse(Interface):
    """A response for a ticket in the Ticketbox"""

    text = Attribute("Text of this response")
    rendered_text = Attribute("Rendered text (html) for caching")
    changes = Attribute("Changes made to the issue in this response.")
    creator = Attribute("Id of user making this change.")
    date = Attribute("Date (plus time) this response was made.")
    type = Attribute("Type of response (additional/clarification/reply).")
    mimetype = Attribute("Mime type of the response.")
    attachment = Attribute("File attachment.")

    def add_change(id_, name, before, after):
        """Add change to the list of changes.
        """


class ICreateResponse(Interface):
    """Create a response"""
    pass


class ITicketFolderView(Interface):
    """Abstract a Ticketbox into a folder for tickets.
    """

    def getFilteredIssues(criteria, **kwargs):
        """Get the contained issues in the given criteria.
        """

    def getIssueSearchQueryString(criteria, **kwargs):
        """Return a query string for an issue query.
        """

    def buildIssueSearchQuery(criteria, **kwargs):
        """Build canonical query for issue search.
        """

    def getMyIssues(openStates, memberId, manager):
        """Get a catalog query result set of my issues.

        So: all issues assigned to or submitted by the current user,
        with review state in openStates.

        If manager is True, you can add more states.
        """

    def getOrphanedIssues(openStates, memberId):
        """Get a catalog query result set of orphaned issues.

        Meaning: all open issues not assigned to anyone and not owned
        by the given user.
        """


class IResponseAdder(IViewletManager):

    mimetype = Attribute("Mime type for response.")
    use_wysiwyg = Attribute("Boolean: Use kupu-like editor.")

    def states_for_display():
        """Get the available transitions for this issue.
        """

    def severities_for_display():
        """Get the available severities for this issue.
        """

    def releases_for_display():
        """Get the releases from the project.
        """

    def managers_for_display():
        """Get the tracker managers.
        """

directlyProvides(IResponseAdder, ITALNamespaceData)


class ITicketReferenceStartupDirectory(Interface):
    """Adapter for getting the startup directory for references in the
    current ticket.
    """

    def __init__(ticket, request):
        """Adapts the ticket and the request.
        """

    def get():
        """Returns the startup directory path.
        """


class IAttachment(Interface):
    """Marker interface for a TicketAttachment"""



from Acquisition import aq_inner, aq_parent
from izug.ticketbox.adapters import ResponseContainer
from izug.ticketbox.handlers import generate_datagrid_column_id
from izug.ticketbox.tests.data import AREA_DATA
from izug.ticketbox.tests.data import PRIORITY_DATA
from izug.ticketbox.tests.data import RELEASE_DATA
from izug.ticketbox.tests.data import STATE_DATA
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles


def login_as_manager(portal):
    setRoles(portal, TEST_USER_ID, ['Manager'])
    login(portal, TEST_USER_NAME)


def logout_manager(portal):
    setRoles(portal, TEST_USER_ID, ['Member'])
    logout()


def create_ticketbox(container, id_='ticketbox'):
    ticketbox = container.get(container.invokeFactory('Ticket Box', id_))

    ticketbox.getField('title').set(ticketbox, "Ticket Box Title")
    ticketbox.getField('description').set(
        ticketbox, "A TicketBox description")

    ticketbox.setIndividualIdentifier("ABC123")
    ticketbox.setAvailableStates(STATE_DATA)
    ticketbox.setAvailableReleases(RELEASE_DATA)
    ticketbox.setAvailableAreas(AREA_DATA)
    ticketbox.setAvailablePriorities(PRIORITY_DATA)

    generate_datagrid_column_id(ticketbox, None)
    ticketbox.reindexObject()

    return ticketbox


def remove_obj(obj):
    parent = aq_parent(aq_inner(obj))
    parent.manage_delObjects([obj.id])


def create_ticket(ticketbox, id_=None, data_index=0):
    id_ = id_ or 'ticket%i' % data_index
    ticket = ticketbox.get(ticketbox.invokeFactory('Ticket', id_))

    ticket.getField('title').set(ticket, "Ticket Title")
    ticket.getField('description').set(ticket, "A Ticket description")
    ticket.getField('state').set(ticket, STATE_DATA[data_index]['id'])
    ticket.getField('area').set(ticket, AREA_DATA[data_index]['id'])
    ticket.getField('priority').set(ticket, PRIORITY_DATA[data_index]['id'])
    ticket.getField('releases').set(ticket, RELEASE_DATA[data_index]['id'])
    ticket.getField('responsibleManager').set(ticket, "test_user_1_")

    ticket.reindexObject()
    ResponseContainer(ticket)

    return ticket



from ftw.upgrade import UpgradeStep


class MigrateDescriptionToTicketDescription(UpgradeStep):
    """Migrate description to ticket description.
    """

    def __call__(self):
        query = {'portal_type': 'Ticket'}
        for ticket in self.objects(query, 'Migrate Ticket description'):
            description = ticket.getRawDescription()
            ticket.Schema()['ticket_description'].set(ticket, description)
            ticket.setDescription('')



# * coding: utf8 *
from datetime import datetime
from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testing import freeze
from izug.ticketbox.testing import TICKETBOX_INTEGRATION_TESTING
from izug.ticketbox.tests import helpers
from unittest2 import TestCase
from xlrd import open_workbook


class TestXlsxExport(TestCase):

    layer = TICKETBOX_INTEGRATION_TESTING

    def setUp(self):
        super(TestXlsxExport, self).setUp()

        portal = self.layer['portal']
        helpers.login_as_manager(portal)

    def test_xlsx_export(self):
        ticketbox = create(Builder('ticket box')
                           .titled(u'My Tickät Box'))

        ticket_builder = Builder('ticket')
        ticket_builder.titled(u'Täst')
        ticket_builder.with_id('1')
        ticket_builder.having(dueDate=DateTime(2010, 11, 10, 17, 14, 35),
                              ticket_description="<p>This is a<br />test.</p>")
        ticket_builder.within(ticketbox)

        with freeze(datetime(2010, 11, 1, 8, 25, 20)):
            create(ticket_builder)

        export_view = ticketbox.restrictedTraverse('tickets_export_xlsx')
        response = export_view()

        workbook = open_workbook(file_contents=response)
        sheet = workbook.sheet_by_index(0)

        data = map(sheet.row_values, range(sheet.nrows))
        headers = data.pop(0)
        data = map(
            lambda row: dict(zip(headers, row)),
            data
        )
        self.maxDiff = None
        self.assertEquals(
            first=[
                {
                    u'Created': u'01.11.2010 08:25',
                    u'Due date': u'10.11.2010 17:14',
                    u'No.': u'1',
                    u'Description': u' This is a test. ',
                    u'Releases': u'',
                    u'Creator': u'test_user_1_',
                    u'Responsible': u'',
                    u'Watched release': u'',
                    u'Priorities': u'',
                    u'State': u'',
                    u'Variety': u'',
                    u'Title': u'Täst',
                    u'Area': u'',
                }
            ],
            second=data
        )



from AccessControl import getSecurityManager
from DateTime import DateTime
from izug.ticketbox.interfaces import ITicket, IResponseContainer, IResponse
from persistent import Persistent
from persistent.list import PersistentList
from zope.annotation.interfaces import IAnnotations
from zope.app.container.contained import ObjectAddedEvent
from zope.app.container.contained import ObjectRemovedEvent
from zope.app.container.interfaces import UnaddableError
from zope.component import adapts
from zope.event import notify
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from zope.app.component import hooks
from Acquisition import aq_inner, aq_parent
from zope.app.component.hooks import getSite


class ResponseContainer(Persistent):

    implements(IResponseContainer)
    adapts(ITicket)
    ANNO_KEY = 'izug.ticketbox.responses'

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(self.context)
        self.__mapping = annotations.get(self.ANNO_KEY, None)
        if self.__mapping is None:
            self.__mapping = PersistentList()
            annotations[self.ANNO_KEY] = self.__mapping

    def __contains__(self, key):
        '''See interface IReadContainer

        Taken from zope.app.container.btree.

        Reimplement this method, since has_key() returns the key if available,
        while we expect True or False.

        >>> c = ResponseContainer()
        >>> "a" in c
        False
        >>> c["a"] = 1
        >>> "a" in c
        True
        >>> "A" in c
        False
        '''
        return key in self.__mapping

    has_key = __contains__

    def __getitem__(self, i):
        i = int(i)
        return self.__mapping.__getitem__(i)

    def __delitem__(self, item):
        self.__mapping.__delitem__(item)

    def __len__(self):
        return self.__mapping.__len__()

    def __setitem__(self, i, y):
        self.__mapping.__setitem__(i, y)

    def append(self, item):
        self.__mapping.append(item)

    def remove(self, id_):
        """Remove item 'id_' from the list.

        We don't actually remove the item, we just set it to None,
        so that when you edit item 3 out of 3 and someone deletes
        item 2 you are not left in the water.

        Note that we used to get passed a complete item, not an id.
        """
        id_ = int(id_)
        self[id_] = None

    def add(self, item):
        if not IResponse.providedBy(item):
            raise UnaddableError(self, item,
                                 "IResponse interface not provided.")
        self.append(item)
        id_ = str(len(self))
        event = ObjectAddedEvent(item, newParent=self.context, newName=id_)
        notify(event)

    def delete(self, id_):
        # We need to fire an ObjectRemovedEvent ourselves here because
        # self[id_].__parent__ is not exactly the same as self, which
        # in the end means that __delitem__ does not fire an
        # ObjectRemovedEvent for us.
        #
        # Also, now we can say the oldParent is the issue instead of
        # this adapter.
        event = ObjectRemovedEvent(self[id_], oldParent=self.context,
                                   oldName=id_)
        self.remove(id_)
        notify(event)


class Response(Persistent):

    implements(IResponse)

    def __init__(self, text):
        self.__parent__ = self.__name__ = None
        self.text = text
        self.changes = PersistentList()
        sm = getSecurityManager()
        user = sm.getUser()
        self.creator = user.getId() or '(anonymous)'
        self.date = DateTime()
        self.type = 'additional'
        self.mimetype = ''
        self.rendered_text = None
        self.attachment = None
        self.references = []

    def add_change(self, id_, name, before, after):
        """Add a new issue change.
        """
        delta = dict(
            id=id_,
            name=name,
            before=before,
            after=after)
        self.changes.append(delta)

    def creator_fullname(self):
        site = getSite()
        member = site.portal_membership.getMemberById(self.creator)
        # In case of Creator no longer exists
        if member is None:
            return self.creator
        fullname = member.getProperty('fullname', member.getId())
        if fullname:
            return fullname
        return member.getId()


class EmptyExporter(object):

    def __init__(self, context):
        self.context = context

    def export(self, export_context, subdir, root=False):
        return


class TicketBoxSubjectCreator(object):

    def __init__(self, context):
        self.context = aq_inner(context)
        self.request = self.context.REQUEST

    def __call__(self, object_):
        site = hooks.getSite()
        portal_properties = getToolByName(object_, 'portal_properties')
        default_subject = '[%s] Notification:' % (site.Title())
        subject = None
        try:
            sheet = portal_properties['ftw.notification-properties']

        except AttributeError:
            subject = default_subject

        else:
            individual_identifier = object_.getIndividualIdentifier()
            if individual_identifier:
                individual_identifier = ' [%s]' % individual_identifier
            subject = '%s%s %s' % (
                sheet.getProperty('notification_email_subject',
                                  default_subject),
                individual_identifier,
                object_.Title())

        return subject


class TicketSubjectCreator(object):

    def __init__(self, context):
        self.context = aq_inner(context)
        self.request = self.context.REQUEST

    def __call__(self, object_):
        site = hooks.getSite()
        portal_properties = getToolByName(object_, 'portal_properties')
        default_subject = '[%s] Notification:' % (site.Title())
        subject = None
        try:
            sheet = portal_properties['ftw.notification-properties']
        except AttributeError:
            subject = default_subject

        else:
            individual_identifier = aq_parent(
                self.context).getIndividualIdentifier()
            if individual_identifier:
                individual_identifier = ' [%s]' % individual_identifier

            subject = '%s%s #%s - %s' % (
                sheet.getProperty('notification_email_subject',
                                  default_subject),
                individual_identifier,
                object_.getId(),
                object_.Title())

        return subject



from ftw.upgrade import UpgradeStep
from izug.ticketbox.content.attachment import TicketAttachment
import logging


LOG = logging.getLogger('ftw.workspace.upgrades')


class UpdateTicketAttachmentClass(UpgradeStep):

    def __call__(self):
        self.update_fti()
        self.migrate_existing_attachments()

    def update_fti(self):
        LOG.info('TabbedViewFolder FTI: update factory')
        self.setup_install_profile(
            'profile-izug.ticketbox.upgrades:4200')

    def migrate_existing_attachments(self):
        query = {'portal_type': 'TicketAttachment'}
        for obj in self.objects(query, "Migrate TicketAttachment class"):
            self.migrate_class(obj, TicketAttachment)



from datetime import datetime
from DateTime import DateTime
from ftw.builder import create, Builder
from ftw.testbrowser import browsing
from ftw.testing import freeze
from izug.ticketbox.testing import TICKETBOX_FUNCTIONAL_TESTING
from izug.ticketbox.tests import helpers
from StringIO import StringIO
from unittest2 import TestCase
from xlrd import open_workbook
from zipfile import ZipFile


class TestZipExport(TestCase):

    layer = TICKETBOX_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestZipExport, self).setUp()

        portal = self.layer['portal']
        helpers.login_as_manager(portal)

        self.ticketbox = create(Builder('ticket box').titled(u'Ticketbox'))

    @browsing
    def test_xls_in_zip_file(self, browser):
        ticket_builder = Builder('ticket')
        ticket_builder.titled(u'T\xc3st')
        ticket_builder.with_id('1')
        ticket_builder.having(dueDate=DateTime(2011, 11, 10, 17, 14, 35),
                              ticket_description='<p>This is a<br />test.</p>')
        ticket_builder.within(self.ticketbox)

        with freeze(datetime(2011, 2, 3, 5, 7, 11)):
            create(ticket_builder)

        browser.login()
        browser.visit(self.ticketbox, view='zip_export')
        self.assertEquals('application/zip', browser.headers['Content-Type'])

        zipfile = ZipFile(StringIO(browser.contents))
        self.assertEquals(['ticketbox.xlsx'], zipfile.namelist())

        xlsx = zipfile.read('ticketbox.xlsx')

        workbook = open_workbook(file_contents=xlsx)
        sheet = workbook.sheet_by_index(0)

        data = map(sheet.row_values, range(sheet.nrows))
        headers = data.pop(0)
        data = map(
            lambda row: dict(zip(headers, row)),
            data
        )
        self.maxDiff = None
        self.assertEquals(
            first=[
                {
                    u'Created': u'03.02.2011 05:07',
                    u'Due date': u'10.11.2011 17:14',
                    u'No.': u'1',
                    u'Description': u' This is a test. ',
                    u'Releases': u'',
                    u'Creator': u'test_user_1_',
                    u'Responsible': u'',
                    u'Watched release': u'',
                    u'Priorities': u'',
                    u'State': u'',
                    u'Variety': u'',
                    u'Title': u'T\xc3st',
                    u'Area': u'',
                }
            ],
            second=data
        )

    @browsing
    def test_ticket_attachment_in_zip_file(self, browser):
        ticket = create(Builder('ticket')
                        .titled(u'Ticket')
                        .within(self.ticketbox))

        file_ = StringIO('GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00'
                         '\x00\x00\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00'
                         '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
        file_.filename = 'im\xc3\xa4ge.gif'
        file_.content_type = 'image/gif'

        browser.login()
        browser.visit(ticket, view='edit')
        browser.fill({'attachment_file': file_}).submit()

        browser.visit(self.ticketbox, view='zip_export')
        zipfile = ZipFile(StringIO(browser.contents))
        image = zipfile.read(u'Ticket/image.gif')
        self.assertEqual(file_.getvalue(), image)



from ftw.tabbedview.browser.listing import CatalogListingView
from ftw.table import helper
from izug.ticketbox import ticketboxMessageFactory as _
from zope.i18n import translate


class BaseTicketListingTab(CatalogListingView):
    """base tabbedview tab for listing tickets.
    """

    types = ['Ticket']

    show_selects = False
    show_menu = False

    sort_on = 'created'
    sort_reverse = True

    def __init__(self, *args, **kwargs):
        super(BaseTicketListingTab, self).__init__(*args, **kwargs)

        self._cached_ticketbox_options = None
        self._state_map = None
        self._priority_map = None
        self._area_map = None

    @property
    def columns(self):
        return ({'column': 'getId',
                 'column_title': _(u"Id"),
                 'sort_index': 'sortable_id',
                 },

                {'column': 'Title',
                 'column_title': _(u"Description"),
                 'sort_index': 'sortable_title',
                 'transform': helper.linked_without_icon,
                 },

                {'column': 'getResponsibleManager',
                 'column_title': _(u"responsibleManager"),
                 'sort_index': 'sortable_responsibleManager',
                 'transform': self.readable_author_helper,
                 },

                {'column': 'getState',
                 'column_title': _(u"State"),
                 'transform': self.state_helper,
                 },

                {'column': 'getDueDate',
                 'column_title': _(u"DueDate"),
                 'transform': helper.readable_date_time_text,
                 },

                {'column': 'getPriority',
                 'column_title': _(u"Priority"),
                 'transform': self.priority_helper,
                 },

                {'column': 'getArea',
                 'column_title': _(u"Area"),
                 'transform': self.area_helper,
                 })

    def readable_author_helper(self, item, userid):
        if userid == '(UNASSIGNED)':
            return translate(_(u'None'), context=self.request)

        else:
            return helper.readable_author(item, userid)

    def _get_ticketbox_path_for(self, item):
        """Returns the ticketbox path for an item.
        """
        return '/'.join(self.context.getPhysicalPath())

    def _get_cached_options_for(self, item, getter_name):
        """Returns all available options for the field with the
        getter name `getter_name` of the ticket box of the
        current item.
        """

        if self._cached_ticketbox_options is None:
            self._cached_ticketbox_options = {}

        box_path = self._get_ticketbox_path_for(item)
        if box_path not in self._cached_ticketbox_options:
            self._cached_ticketbox_options[box_path] = {}
        box_cache = self._cached_ticketbox_options[box_path]

        if getter_name not in box_cache:
            box_cache[getter_name] = {}

            box = self.context.unrestrictedTraverse(box_path)
            for option in getattr(box, getter_name)():
                box_cache[getter_name][option['id']] = option['title']

        return box_cache[getter_name]

    def state_helper(self, item, stateid):
        options = self._get_cached_options_for(item, 'getAvailableStates')
        return options.get(stateid)

    def priority_helper(self, item, priorityid):
        options = self._get_cached_options_for(item, 'getAvailablePriorities')
        return options.get(priorityid)

    def area_helper(self, item, areaid):
        options = self._get_cached_options_for(item, 'getAvailableAreas')
        return options.get(areaid)



from unittest2 import TestCase
from ftw.testing.layer import ComponentRegistryLayer
from izug.ticketbox.utils import uniquify_ids
from izug.ticketbox.utils import create_uniqe_id


class NormalizerLayer(ComponentRegistryLayer):

    def setUp(self):
        super(NormalizerLayer, self).setUp()

        import plone.i18n.normalizer
        self.load_zcml_file('configure.zcml', plone.i18n.normalizer)


NORMALIZER_FIXTURE = NormalizerLayer()


class TestUniquifyIds(TestCase):

    layer = NORMALIZER_FIXTURE

    def test_new_ids_set(self):
        input = [{'id': '', 'title': 'Foo'},
                 {'id': '', 'title': 'Bar'}]

        output = [{'id': 'foo', 'title': 'Foo'},
                  {'id': 'bar', 'title': 'Bar'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_partical_ids(self):
        input = [{'id': 'foo', 'title': 'Foo'},
                 {'id': '', 'title': 'Bar'}]

        output = [{'id': 'foo', 'title': 'Foo'},
                  {'id': 'bar', 'title': 'Bar'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_keeps_unexpected_ids(self):
        input = [{'id': 'fooX', 'title': 'Foo'},
                 {'id': '', 'title': 'Bar'}]

        output = [{'id': 'fooX', 'title': 'Foo'},
                  {'id': 'bar', 'title': 'Bar'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_conflicting_ids(self):
        input = [{'id': '', 'title': 'Foo'},
                 {'id': '', 'title': 'Foo'},
                 {'id': '', 'title': 'Foo'}]

        output = [{'id': 'foo', 'title': 'Foo'},
                  {'id': 'foo-1', 'title': 'Foo'},
                  {'id': 'foo-2', 'title': 'Foo'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_modification_keeps_ids(self):
        input = [{'id': '', 'title': 'Foo'},
                 {'id': 'foo', 'title': 'Bar'}]

        output = [{'id': 'foo-1', 'title': 'Foo'},
                  {'id': 'foo', 'title': 'Bar'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_keeps_other_keys(self):
        input = [{'id': 'foo', 'title': 'Foo', 'bar': 'Bar'}]
        output = [{'id': 'foo', 'title': 'Foo', 'bar': 'Bar'}]

        self.assertEqual(uniquify_ids(input), output)

    def test_does_not_copy(self):
        input = [{'id': '', 'title': 'Foo'}]
        self.assertEqual(id(uniquify_ids(input)), id(input))


class TestCreateUniqueId(TestCase):

    layer = NORMALIZER_FIXTURE

    def test_first_id(self):
        self.assertEqual(create_uniqe_id('Foo', []), 'foo')

    def test_existing_id(self):
        self.assertEqual(create_uniqe_id('Foo', ['foo']), 'foo-1')
        self.assertEqual(create_uniqe_id('Foo', ['foo', 'foo-1']), 'foo-2')

    def test_spaces(self):
        self.assertEqual(create_uniqe_id('Foo bar', []), 'foo-bar')



from izug.ticketbox.adapters import Response
from izug.ticketbox.interfaces import IResponseContainer
from izug.ticketbox.testing import TICKETBOX_INTEGRATION_TESTING
from izug.ticketbox.tests import helpers
from unittest2 import TestCase


class TestResponse(TestCase):

    layer = TICKETBOX_INTEGRATION_TESTING

    def setUp(self):
        super(TestResponse, self).setUp()

        portal = self.layer['portal']
        helpers.login_as_manager(portal)

        self.ticketbox = helpers.create_ticketbox(portal)
        self.ticket1 = helpers.create_ticket(self.ticketbox)
        self.ticket2 = helpers.create_ticket(self.ticketbox, data_index=1)

        self.container1 = IResponseContainer(self.ticket1)
        self.container2 = IResponseContainer(self.ticket2)

    def tearDown(self):
        helpers.remove_obj(self.ticketbox)
        helpers.logout_manager(self.layer['portal'])
        super(TestResponse, self).tearDown()

    def test_add_and_remove(self):
        response1 = Response('response1')
        response2 = Response('response3')
        response3 = Response('response3')

        self.assertEquals(len(self.container1), 0)
        self.assertEquals(len(self.container2), 0)

        self.container1.add(response1)
        self.container1.add(response2)
        self.container2.add(response3)

        self.assertEquals(len(self.container1), 2)
        self.assertEquals(len(self.container2), 1)

        self.container1.remove(0)
        self.container1.remove(1)

        self.assertEquals(response1 in self.container1, False)
        self.assertEquals(response2 in self.container1, False)
        self.assertEquals(response3 in self.container2, True)




# # -*- coding: utf-8 -*-
from Products.statusmessages.interfaces import IStatusMessage
from izug.ticketbox import ticketboxMessageFactory as _
from izug.ticketbox.utils import uniquify_ids
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import queryUtility


def generate_datagrid_column_id(obj, event):
    """Generate column-ids for all datagrids in the ticketbox."""

    uniquify_ids(obj.getAvailableStates())
    uniquify_ids(obj.getAvailableReleases())
    uniquify_ids(obj.getAvailablePriorities())
    uniquify_ids(obj.getAvailableAreas())
    uniquify_ids(obj.getAvailableVarieties())


def move_document_to_reference(obj, event):
    """Create own File and add it to References"""

    file_ = obj.getAttachment()
    if file_.data != '':
        new_id = queryUtility(IIDNormalizer).normalize(
            file_.filename.decode('utf-8'))
        if obj.get(new_id, None):
            IStatusMessage(obj.REQUEST).addStatusMessage(
                _(u"text_file_exists_error"), type='error')
            obj.setAttachment('DELETE_FILE')
            return
        new_file_id = obj.invokeFactory(
            type_name="TicketAttachment",
            id=new_id,
            title=file_.filename,
            file=file_)
        new_file = obj.get(new_file_id, None)
        if new_file is None:
            return
        uid = new_file.UID()
        references = obj.getRawAttachments()

        if not isinstance(references, list):
            references = [references]

        references.append(uid)
        obj.setAttachments(references)
        obj.setAttachment('DELETE_FILE')
        obj.reindexObject()



from Acquisition import aq_inner, aq_parent
from izug.ticketbox.browser.helper import readable_author
from izug.ticketbox.interfaces import ITicket
from izug.ticketbox.interfaces import ITicketBox
from plone.indexer.decorator import indexer
from Products.ATContentTypes.interface.file import IATFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_callable
from Products.CMFPlone.utils import safe_unicode
import re


num_sort_regex = re.compile('\d+')


@indexer(ITicketBox)
def get_owner_index(obj):
    userid = obj.getOwner(0).getId()
    if not userid:
        userid = obj.Creator
    if callable(userid):
        userid = userid()
    return userid or ''


@indexer(ITicket)
def get_ticket_description(obj):
    pt = getToolByName(obj, 'portal_transforms')
    description = obj.getTicket_description()
    return pt.convertTo('text/plain', description).getData()


@indexer(ITicket)
def sortable_id(obj):
    """put any zeros before the id
    so its possible to sort the id correctly
    """

    _id = getattr(obj, 'getId', None)
    if _id is not None:
        if safe_callable(_id):
            _id = _id()
        if isinstance(_id, basestring):
            value = _id.lower().strip()
            # Replace numbers with zero filled numbers
            value = num_sort_regex.sub(zero_fill, value)
            # Truncate to prevent bloat
            value = safe_unicode(value)[:40].encode('utf-8')
            return value
    return ''


@indexer(IATFile)
def sortable_ticket_references(obj):
    """Get ticket nr based on parent
    """
    parent = obj.aq_inner.aq_parent
    if ITicket.providedBy(parent):
        url = obj.absolute_url()
        split_url = url.split("/")
        if len(split_url) >= 2:
            return num_sort_regex.sub(zero_fill, split_url[-2])


def zero_fill(matchobj):
    return matchobj.group().zfill(8)


@indexer(ITicket)
def sortable_responsibleManager(obj):
    """get the fullname of the author to sort correctly"""
    author = readable_author(obj)
    if isinstance(author, unicode):
        author = author.encode('utf-8')
    return author


@indexer(ITicket)
def ticketbox_title(obj):
    """Indexer for indexing the ticketbox title on a ticket.
    """

    parent = aq_parent(aq_inner(obj))
    return parent.Title()



STATE_DATA = [
    {'id': 'test_id_1',
     'title': "Show All Tickets",
     'show_in_all_tickets': '1',
     'show_in_my_tickets': '0',
     },

    {'id': 'test_id_2',
     'title': "Show My Tickets",
     'show_in_all_tickets': '0',
     'show_in_my_tickets': '1',
     },

    {'id': '',
     'title': "Test Id State",
     'show_in_all_tickets': '1',
     'show_in_my_tickets': '1',
     }]


AREA_DATA = [
    {'id': u'test_id_1',
     'title': u'Internet',
     },

    {'id': u'test_id_2',
     'title': u'Intranet',
     },

    {'id': u'',
     'title': u'Test Id Area',
     }]


RELEASE_DATA = [
    {'id': u'version-1',
     'title': u'Version 1',
     },

    {'id': u'version-2',
     'title': u'Version 2',
     },

    {'id': u'',
     'title': u'Test Id Release',
     }]


PRIORITY_DATA = [
    {'id': u'test_id_1',
     'title': u'High',
     },

    {'id': u'test_id_2',
     'title': u'Low',
     },

    {'id': u'',
     'title': u'Test Id Priority',
     }]


RESPONSIBLE_DATA = [
    {'id': u'testuser',
     'title': u'Testuser',
     },

    {'id': u'admin',
     'title': u'Admin',
     }]
